-- ===========================================================================
-- CrystalCounter 1.1 - Plugin for Vendetta Online (all platforms)
-- ===========================================================================
-- Created by Snib in May 2022
--
-- Counts the crystals you earn and stores them in a database.
-- Every time you earn crystals it will print your current daily total.
-- You an also check your daily earnings via the command
--     /crystalCount
-- To see your earnings history since using this plugin, use the command
--     /crystalHistory
-- ===========================================================================

local crystalDatabase = {}
local notesId = 0

local function cprint(text)
	print("\127ffffff[\12700ff00CC\127o] " .. text .. "\127o")
end

local function init()
	local charId = GetCharacterID()
	if charId then
		notesId = charId .. "135790"
		crystalDatabase = unspickle(LoadSystemNotes(notesId) or crystalDatabase)
		return true
	end
	cprint("CrystalCounter could not be initialized, please log out and back in.")
	return false
end

local function countCrystals()
	if notesId == 0 and not init() then return end
	cprint("You earned " .. (crystalDatabase[os.date("%Y-%m-%d")] or 0) ..
		" " .. CrystalIcon .. " today.")
end

local function chatHandler(_, chat)
	if chat.missionid == 0 then
		local match = string.match(chat.msg,
			"\\*\\*\\*.-(%d+)" .. CrystalIcon)
		if match then
			local today = os.date("%Y-%m-%d")
			if notesId == 0 and not init() then return end
			crystalDatabase[today] = (
				crystalDatabase[today] or 0) + tonumber(match)
			SaveSystemNotes(spickle(crystalDatabase), notesId)
			countCrystals()
		end
	end
end

local function showDatabase()
	local iupList = iup.stationsubsublist { expand = "YES" }
	local close_button = iup.stationbutton {
		title = "Close",
		expand = "HORIZONTAL",
		action = function() return iup.CLOSE end
	}
	local dialog = iup.dialog {
		iup.vbox {
			iupList,
			close_button,
		},
		title = "Crystal Earnings History",
		size = tostring(Font.Default * 16) .. "xTHREEQUARTER",
		topmost = "YES",
		defaultesc = close_button
	}
	if notesId == 0 and not init() then return end
	local dates = {}
	for date in pairs(crystalDatabase) do
		table.insert(dates, date)
	end
	table.sort(dates, function(a, b) return a > b end)
	local row = 1
	for i = 1, #dates do
		iupList[row] = "On " .. dates[i] .. " you earned " ..
			crystalDatabase[ dates[i] ] .. CrystalIcon
		row = row + 1
	end
	if HUD.visible then
		-- when the HUD is shown, show as a popup to prevent clicking on
		-- the HUD and bringing it to the front
		dialog:popup(iup.CENTER, iup.CENTER)
	else
		dialog:show()
	end
end

local function logout()
	notesId = 0
end

RegisterEvent(chatHandler, "CHAT_MSG_SERVER")
RegisterEvent(logout, "PLAYER_LOGGED_OUT")
RegisterUserCommand("crystalCount", countCrystals)
RegisterUserCommand("crystalHistory", showDatabase)
